#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files

short _ti89;                  // Produce .89Z File

#define ARROWS_ROW 0xfffe      // Constants for low-level keyboard access
#define LEFT_KEY 2
#define RIGHT_KEY 8
#define UP_KEY 1
#define DOWN_KEY 4
#define SECOND_KEY 16
#define ESC_ROW 0xffbf
#define ESC_KEY 1
#define DIAMOND_KEY 64

#define TRUE 1
#define FALSE 0

struct Highscore {
	char initials[5][4];
	int score[5];
};

static struct Highscore highscore={{"JJH", "JJH", "JJH", "JJH", "JJH"}, {0,0,0,0,0}};
char diamond_pressed=FALSE;

void PutSprite16 (int x, int y, int h, int *sprite, void *buff)
{
  register long addr = (long)buff + 30*y + ((x>>3) & 0xFFFE);
  register int i, cnt = 16 - (x&15);
  for (i = 0; i < h; addr += 30) *(long*)addr ^= (long)(unsigned)sprite[i++] << cnt;
}

void DispHighscore(void)
{
	int count;
	clrscr();
	printf("Highscores:\n\n");
	for(count=0;count<=4;count++)
		printf("%d. %d by %s\n",count+1,highscore.score[count],highscore.initials[count]);
	return;
}

void end(int score)
{

	int count,key=0,counter,counters;
	
	clrscr();
	printf("%|26s","Game Over");
	printf("\n\nYour score is %d.",score);
	if(score<=highscore.score[4])
		DispHighscore();
	else //score>highscore.score[4]
	{
		for(counter=4;score>highscore.score[counter] && counter>=0;counter--)
			;
		counter++;
		printf("\nOld Highscore was %d by %s",highscore.score[counter], highscore.initials[counter]); 
		for(count=4;count>counter;count--)
		{
			highscore.score[count]=highscore.score[count-1];
			strcpy(highscore.initials[count],highscore.initials[count-1]);
		}
		highscore.score[counter]=score;
		printf("\nNew Highscore of %d!",score);
		printf("\nEnter your initials.\n");
		for(count=0;count<=2;count++)
		{
			GKeyFlush();
			while((key=GKeyIn(NULL,0))==0)
			{
				GKeyFlush();
				ST_busy(ST_IDLE);
			}
			printf("%c",key);
			highscore.initials[counter][count]=key;
			if(key==257)
			{
				printf("\n");
				for(counters=0;counters<count-1;counters++)
					printf("%c",highscore.initials[counter][counters]);
				count-=2;
			}	
		}
		highscore.initials[counter][3]='\0';
	}
	GKeyFlush();
	while(GKeyIn(NULL,0)!=13);
		GKeyFlush();
	clrscr();
}

int move_ball(int x,int y,int which,int how_many)
{
	int  ball[16] = { 0x0000,
	0x0000,
	0x0000,
	0x0000,
	0x03C0,
	0x07E0,
	0x0FF0,
	0x0FF0,
	0x0FF0,
	0x0FF0,
	0x07E0,
	0x03C0,
	0x0000,
	0x0000,
	0x0000,
	0x0};

	if(which==1)
	{
		PutSprite16(x,y,16,ball,LCD_MEM);
		return y;
	}
	DrawIcon(x,y,ball,A_XOR);
	y+=how_many;
	PutSprite16(x,y,16,ball,LCD_MEM);
	return y;
	        	
}
int move_ship(int x,int y,int what,int how_many)
{
	int key=0;
	int ship[16] = { 0x0180,
	0x03C0,
	0x07E0,
	0x0FF0,
	0x1FF8,
	0x3FFC,
	0x7FFE,
	0xFFFF,
	0x0000,
	0x0000,
	0x0000,
	0x0000,
	0x0000,
	0x0000,
	0x0000,
	0x0};

	if(what==1)
	{
		PutSprite16(x,y,16,ship,LCD_MEM);
		return x;
	}
	ST_busy(ST_IDLE);
	key=_rowread(ARROWS_ROW);
	DrawIcon(x,y,ship,A_XOR);
	x+=(((x<142)&&(key&RIGHT_KEY))-((x>2)&&(key&LEFT_KEY)))*how_many;
	PutSprite16(x,y,16,ship,LCD_MEM);
	if(key&SECOND_KEY)
		off();
	if(key&DIAMOND_KEY)
		diamond_pressed=TRUE;
	return x;
}
		
int engine(int num_of_balls,int *ball_x,int *ball_y,int *ship_x,int *score)
{
	int ball_num,count,counter;
	char bar[50];
	char diamond_pressed_already=FALSE;
	
	for(counter=1;counter<=10;counter++)
	{
	  ball_num=0;
		ball_y[0]=-4;
		ball_x[0]=*ship_x;
		move_ball(ball_x[0],ball_y[0],1,num_of_balls);
		for(count=1;count<=num_of_balls-1;count++)
		{
	    ball_y[count]=-4;
			ball_x[count]=random(148);
			move_ball(ball_x[count],ball_y[count],1,num_of_balls);
		}    
		pushkey(13);
		for(count=0;count<=88;count+=num_of_balls)
		{
			for(ball_num=0;ball_num<=num_of_balls-1;ball_num++)
			{
				if((ball_x[ball_num]+12>=(*ship_x)+4 && ball_x[ball_num]+4<=(*ship_x)+12) && (ball_y[ball_num]+12>=84 && ball_y[ball_num]+8<=84+8))
				{
					end(*score);
					return 1;
				}
				ball_y[ball_num]=move_ball(ball_x[ball_num],ball_y[ball_num],0,num_of_balls);
			}
			*ship_x=move_ship(*ship_x,84,0,num_of_balls);
			if(diamond_pressed==TRUE)
			{
				diamond_pressed=FALSE;
				if(diamond_pressed_already==FALSE)
				{
					diamond_pressed_already=TRUE;
					break;
				}
			}
			if(_rowread(ESC_ROW)&ESC_KEY)
			{
				end(*score);
				return 1;
			}        
		}
		(*score)+=num_of_balls*10;
		ClrScr();
		move_ship(*ship_x,84,1,2);
		OSSetSR(0x0000);
		sprintf(bar,"Score: %d Highscore: %d by %s",*score,highscore.score[0],highscore.initials[0]);
		ST_helpMsg(bar);
		OSSetSR(0x0700); //Is this necessary??
	}
	return 0;
}
void game(void)
{
	int ball_x[10]={0,0,0,0,0,0,0,0,0,0};
	int ball_y[10]={-4,-4,-4,-4,-4,-4,-4,-4,-4,-4};
	int ship_x=72,ship_y=84,count=0,score=0,ball_num=0,counter=0,num_of_balls;
//	static int highscore=0;
//	static char initials[4]={0,0,0,0};
	char bar[50];
	
	move_ship(ship_x,ship_y,1,1);
	OSSetSR(0x0000);
	sprintf(bar,"Score: %d Highscore: %d by %s",score,highscore.score[0],highscore.initials[0]);
	ST_helpMsg(bar);
	OSSetSR(0x0700); //Is this necessary?
	for(counter=1;counter<=10;counter++)
	{
		ball_y[0]=-4;
		ball_x[0]=ship_x;
		move_ball(ball_x[ball_num],ball_y[ball_num],1,1);
		pushkey(13);
		for(count=0;count<=88;count+=1)
		{
			if((ball_x[ball_num]+12>=ship_x+4 && ball_x[ball_num]+4<=ship_x+12) && (ball_y[ball_num]+12>=ship_y && ball_y[ball_num]+8<=ship_y+8))
			{
				end(score);
				return;
			}
			ship_x=move_ship(ship_x,ship_y,0,1);
	    if(_rowread(ESC_ROW)&ESC_KEY)
	    {
				end(score);
				return;
			}
			ball_y[ball_num]=move_ball(ball_x[ball_num],ball_y[ball_num],0,1);       
		}
		score+=10;
		ClrScr();
		move_ship(ship_x,ship_y,1,1);
		OSSetSR(0x0000);
		sprintf(bar,"Score: %d Highscore: %d by %s",score,highscore.score[0],highscore.initials[0]);
		ST_helpMsg(bar);
		OSSetSR(0x0700); //Is this necessary?
	}
//----------------------------------------;
//2+ balls;
	num_of_balls=2;
	while(1)
	{
		if(1==engine(num_of_balls,&ball_x[0],&ball_y[0],&ship_x,&score))
			return;
		diamond_pressed=FALSE;
		num_of_balls++;
	}
	end(score);
}

void _main(void)
{
	int key=0;
	
	while(1)
	{
		clrscr();
		printf_xy(56,25,"1. Start");
		printf_xy(41,35,"2. Highscores");
		printf_xy(35,45,"3. Instructions");
		printf_xy(53,55,"4. About");
		printf_xy(59,65,"5. Exit");
		key=0;
		GKeyFlush();
		key=GKeyIn(NULL,0);
		switch(key)
		{
			case 50: //2
				DispHighscore();
				GKeyIn(NULL,0);
				break;
			case 51: //3
				clrscr();
				printf("Esc - exits the game or \nreturns to main menu.\n\nArrow Keys - move ship \naround\n\n2nd - pauses game and \nturns off calculator\n\nDiamond - all balls \ndisappear once per level.%c",13);
				GKeyFlush();
				GKeyIn(NULL,0);
				printf("\n\n\nThe object of the game is to avoid being hit by the oncoming balls.");
				GKeyFlush();
				GKeyIn(NULL,0);
				break;
			case 52: //4
				clrscr();
				printf("\nDodge v3.0\nby Josh Hill\n\nProgrammers Anonymous \nMember\nhttp://pa.ticalc.org");
				GKeyFlush();
				GKeyIn(NULL,0);
				break;
			case 53: //5
			case 264: //esc
				return;
				break;
			case 49: //1
				clrscr();
				OSSetSR(0x0700);
				game();
				OSSetSR(0x0000); //Necessary?
				break;
		}
	}
	return;
}
